#include <assert.h>
#include "gc/gc.h"
#include "gc/object.h"
#include "gc/pointer.h"

namespace gc {

void GarbageCollectSystem::markPtrList(Link *ptrs, std::vector<Object*> *marked) {
	for (Link *p = ptrs->next; p != ptrs; p=p->next) {
		PointerBase *b = dynamic_cast<PointerBase*>(p);
		assert(b);
		Object *bo = b->ptrBase();
		if (bo && !bo->marked) {
			bo->marked=true;
			marked->push_back(bo);
		}
	}
}

void GarbageCollectSystem::collect() {

	//1) clear all
	for (Link *p = objs.next; p != &objs; p=p->next) {
		Object *o = dynamic_cast<Object*>(p);
		assert(o);
		o->marked = false;
	}
	//keep track of our current set of all newly-marked in a stl vector
	std::vector<Object*> m1, m2;
	std::vector<Object*> *curObjs = &m1;
	std::vector<Object*> *nextObjs = &m2;
	//2) mark base ptrs
	markPtrList(&ptrs, curObjs);
	//3) propagate
	for(;;) {
		//propagate from all
		for (std::vector<Object*>::iterator i = curObjs->begin(); i != curObjs->end(); ++i) {
			Object *o = *i;
			assert(o);
			//don't clear inside markPtrList cuz it'll be accum'ing even when it has contents
			markPtrList(&o->ptrs, nextObjs);
		}
		
		//swap lists
		std::vector<Object*> *tmp = curObjs;
		curObjs = nextObjs;
		nextObjs = tmp;
		nextObjs->clear();

		//run it until no new nodes are marked
		if (!curObjs->size()) break;
	}
	//4) delete all the unmarked nodes
	Link *next = NULL;
	for (Link *p = objs.next; p != dynamic_cast<Link*>(&objs); p=next) {
		next = p->next;	//store the next ptr here, in case the current gets deleted
		//this is safe so long as no one deletes the next...
		Object *o = dynamic_cast<Object*>(p);
		assert(o);
		if (!o->marked) {
			//Object's dtor will unlink it from the rest of the world...
			delete o;
		}
	}
	
}

Singleton<GarbageCollectSystem> sys;

};
