#include "gc/pointer.h"
#include "gc/object.h"
#include "gc/gc.h"

namespace gc {

PointerBase::PointerBase(Object *_owner, Object *_p)
: owner(_owner), p(_p)
{
	/*
	now here's an abuse of OOP
	Pointerbase doesn't use this at all, only its children
	owner is used for Pointer
	!owner for GlobalPointer
		which maybe shoudln't even be a subclass of Pointer
	*/
	if (owner) {
		attach(&owner->ptrs);
	} else {
		/*
		This (and the ctor below) is one of the only reference in all the library to "sys",
		our singleton for holding global pointers and for performing the
		flood-fill operations.
		I'm toying with the notion of placing this pointer inside each smartptr ... 
		... or at least each global pointer ... (subclass'd, pushing owner to the Pointer child class, putting this in GlobalPointer)
		... (or give Object and System a common parent class (possibly making System an Object))
		... and using its attach here
		*/
		attach(&sys->ptrs);
	}
} 

PointerBase::PointerBase(const PointerBase &o)
: owner(o.owner), p(o.p)
{
	if (owner) {
		attach(&owner->ptrs);
	} else {
		attach(&sys->ptrs);
	}
}

PointerBase::~PointerBase() {
}

PointerBase &PointerBase::operator=(Object *_p) {
	p = _p;
	return *this;
}

PointerBase &PointerBase::operator=(const PointerBase &_p) {
	p = _p.p;
	return *this;
}


};
